package com.njyunzhi.invoice.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.njyunzhi.invoice.common.BaseController;
import com.njyunzhi.invoice.common.Constants;
import com.njyunzhi.invoice.common.DateUtils;
import com.njyunzhi.invoice.common.ResponseBean;
import com.njyunzhi.invoice.entity.TaInvoiceFill;
import com.njyunzhi.invoice.entity.TaPerson;
import com.njyunzhi.invoice.service.ITaInvoiceFillService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.njyunzhi.invoice.service.ITaInvoiceDetailService;
import com.njyunzhi.invoice.entity.TaInvoiceDetail;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
    * 报销明细 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-07-04
 */

@Api(tags = "报销明细")
@RestController
@RequestMapping("/")
public class TaInvoiceDetailController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaInvoiceDetailController.class);

    @Autowired
    public ITaInvoiceDetailService iTaInvoiceDetailService;

    @Autowired
    public ITaInvoiceFillService iTaInvoiceFillService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/invoice/{invoiceId}/detail",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taInvoiceDetailList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
									 @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                     @ApiParam("班级ID") @PathVariable(value ="invoiceId") String invoiceId) throws Exception{

        IPage<TaInvoiceDetail> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaInvoiceDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.gt("status", Constants.STATUS_DELETE);
        queryWrapper.eq("invoice_id",invoiceId);
        queryWrapper.orderByDesc("create_date");
        IPage<TaInvoiceDetail> result = iTaInvoiceDetailService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }

    @RequestMapping(value="/admin/invoice/{invoiceId}/detail",method= RequestMethod.POST)
    @ApiOperation(value="一键生成", notes = "一键生成", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taInvoiceDetailAdd(@ApiParam("班级ID") @PathVariable(value ="invoiceId") String invoiceId) throws Exception{

        if (iTaInvoiceDetailService.createBatch(invoiceId)){
            return ResponseBean.success("ok");
        } else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/invoice/detail/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean taInvoiceDetailDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        if(iTaInvoiceDetailService.removeById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taInvoiceDetail 实体对象
     * @return
     */
    @RequestMapping(value="/admin/invoice/{invoiceId}/detail/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean taInvoiceDetailUpdate(@ApiParam("班级ID") @PathVariable(value ="invoiceId") String invoiceId,
                                              @ApiParam("对象ID") @PathVariable String id,
                                              @ApiParam("更新内容") @RequestBody TaInvoiceDetail taInvoiceDetail) throws Exception{

        taInvoiceDetail.setDetailId(id);
        taInvoiceDetail.setInvoiceId(invoiceId);
        if (iTaInvoiceDetailService.updateById(taInvoiceDetail)){
            return ResponseBean.success(iTaInvoiceDetailService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    @RequestMapping(value="/h5/invoice/{invoiceId}/detail/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="H5确认报销单", notes = "H5确认报销单", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean invoiceConfirm(@ApiParam("班级ID") @PathVariable(value ="invoiceId") String invoiceId,
                                       @ApiParam("对象ID") @PathVariable String id,
                                       @ApiParam("更新内容") @RequestBody TaInvoiceDetail taInvoiceDetail) throws Exception{

        TaPerson taPerson = currentPerson();

        taInvoiceDetail.setDetailId(id);
        taInvoiceDetail.setPersonId(taPerson.getPersonId());
        taInvoiceDetail.setInvoiceId(invoiceId);
        if (iTaInvoiceDetailService.updateById(taInvoiceDetail)){
            return ResponseBean.success(iTaInvoiceDetailService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    private boolean getIsFinished(TaInvoiceFill taInvoiceFill) {
        return Constants.INVOICE_FINISHED == taInvoiceFill.getStatus() ||
                DateUtils.now().isAfter(taInvoiceFill.getEndDate());
    }


    @RequestMapping(value="/h5/invoice/{invoiceId}",method= RequestMethod.GET)
    @ApiOperation(value="获取报销单", notes = "获取报销单", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getInvoice(@ApiParam("班级ID") @PathVariable(value ="invoiceId") String invoiceId) throws Exception{

        TaInvoiceFill taInvoiceFill = iTaInvoiceFillService.getById(invoiceId);
        if (null == taInvoiceFill || taInvoiceFill.getStatus() != Constants.STATUS_NORMAL) {
            return ResponseBean.error("报销单不存在");
        }

        TaPerson taPerson = currentPerson();
        TaInvoiceDetail taInvoiceDetail = iTaInvoiceDetailService.getByPerson(invoiceId, taPerson.getPersonId());
        if (null == taInvoiceDetail) {
            return ResponseBean.error("找不到报销数据或者您不在此次名单中");
        }

        taInvoiceDetail.setIsFinished(getIsFinished(taInvoiceFill));
        return ResponseBean.success(taInvoiceDetail);
    }

    @RequestMapping(value="/h5/invoice/{invoiceId}/detail/{id}",method= RequestMethod.GET)
    @ApiOperation(value="获取报销单详情", notes = "获取报销单详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getDetail(@ApiParam("班级ID") @PathVariable(value ="invoiceId") String invoiceId,
                                       @ApiParam("对象ID") @PathVariable String id) throws Exception{

        TaInvoiceFill taInvoiceFill = iTaInvoiceFillService.getById(invoiceId);
        if (null == taInvoiceFill || taInvoiceFill.getStatus() != Constants.STATUS_NORMAL) {
            return ResponseBean.error("报销单不存在");
        }

        TaPerson taPerson = currentPerson();
        TaInvoiceDetail taInvoiceDetail = iTaInvoiceDetailService.getById(id);
        if (null == taInvoiceDetail || taInvoiceDetail.getStatus() == Constants.STATUS_DELETE || !taPerson.getPersonId().equals(taInvoiceDetail.getPersonId())) {
            return ResponseBean.error("找不到报销数据详情");
        }

        taInvoiceDetail.setIsFinished(getIsFinished(taInvoiceFill));
        return ResponseBean.success(taInvoiceDetail);
    }
}
