package com.njyunzhi.invoice.service.impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.read.listener.PageReadListener;
import com.alibaba.excel.support.ExcelTypeEnum;
import com.njyunzhi.invoice.common.Constants;
import com.njyunzhi.invoice.common.DateUtils;
import com.njyunzhi.invoice.entity.TaInvoiceOrg;
import com.njyunzhi.invoice.entity.TaInvoicePerson;
import com.njyunzhi.invoice.mapper.TaInvoiceOrgMapper;
import com.njyunzhi.invoice.mapper.TaInvoicePersonMapper;
import com.njyunzhi.invoice.service.ITaInvoicePersonService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 报销人员 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2022-07-04
 */
@Service
public class TaInvoicePersonServiceImpl extends BaseServiceImpl<TaInvoicePersonMapper, TaInvoicePerson> implements ITaInvoicePersonService {

    @Autowired
    TaInvoiceOrgMapper taInvoiceOrgMapper;

    @Override
    public List<TaInvoicePerson> getByName(String invoiceId, String invoiceOrgId, String name) {
        return baseMapper.getByName(invoiceId, invoiceOrgId, name);
    }

    @Override
    public boolean createNew(TaInvoicePerson taInvoicePerson) throws Exception {
        TaInvoiceOrg taInvoiceOrg = taInvoiceOrgMapper.getByInvoiceAndOrgName(taInvoicePerson.getInvoiceId(), taInvoicePerson.getOrgName());
        if (null == taInvoiceOrg) {
            taInvoiceOrg = createByPerson(taInvoicePerson);
        }
        taInvoicePerson.setInvoiceOrgId(taInvoiceOrg.getInvoiceOrgId());

        List<TaInvoicePerson> list = getByName(taInvoicePerson.getInvoiceId(), taInvoicePerson.getInvoiceOrgId(), taInvoicePerson.getName());
        if (null != list && list.size() > 0) {
            throw new Exception("人员名称重复");
        }

        taInvoicePerson.setStatus(Constants.STATUS_NORMAL);
        taInvoicePerson.setCreateDate(DateUtils.now());

        return save(taInvoicePerson);
    }

    @Override
    public void batchUpload(String invoiceId, MultipartFile file) throws Exception {
        LocalDateTime now = DateUtils.now();
        EasyExcel.read(file.getInputStream(), TaInvoicePerson.class, new PageReadListener<TaInvoicePerson>(list -> {
            for(TaInvoicePerson person : list) {
                TaInvoiceOrg taInvoiceOrg = taInvoiceOrgMapper.getByInvoiceAndOrgName(invoiceId, person.getOrgName());
                if (null == taInvoiceOrg) {
                    taInvoiceOrg = createByPerson(person);
                }
                person.setInvoiceOrgId(taInvoiceOrg.getInvoiceOrgId());

                List<TaInvoicePerson> exists = getByName(invoiceId, person.getInvoiceOrgId(), person.getName());
                if (null != exists && exists.size() > 0) {
                    continue;
                }

                person.setStatus(Constants.STATUS_NORMAL);
                person.setCreateDate(now);
                baseMapper.insert(person);
            }
        })).excelType(ExcelTypeEnum.XLSX).sheet().headRowNumber(1).doRead();
    }

    private TaInvoiceOrg createByPerson(TaInvoicePerson taInvoicePerson) {
        TaInvoiceOrg taInvoiceOrg = new TaInvoiceOrg();
        taInvoiceOrg.setName(taInvoicePerson.getOrgName());
        taInvoiceOrg.setStatus(Constants.STATUS_NORMAL);
        taInvoiceOrg.setCreateDate(DateUtils.now());

        taInvoiceOrgMapper.insert(taInvoiceOrg);
        return taInvoiceOrg;
    }
}
